@@ -29,14 +29,16 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include "file.h"
+
+#ifndef	lint
+FILE_RCSID("@(#)$File: getopt_long.c,v 1.5 2009/02/03 20:27:51 christos Exp $")
+#endif	/* lint */
+
 #include <assert.h>
 #ifdef HAVE_ERR_H
 #include <err.h>
 #else
-#include <stdio.h>
 #define warnx printf
 #endif
 #include <errno.h>
